@@ -66,6 +66,8 @@ fatal_handler (gpointer unused, int unknown, const gchar *msg)
 void
 gkr_crypto_setup (void)
 {
+	unsigned seed;
+
 	if (gcrypt_initialized)
 		return;
 		
@@ -80,6 +82,9 @@ gkr_crypto_setup (void)
 	                             gkr_secure_free);
 	                             
 	gcrypt_initialized = TRUE;
+	
+	gcry_create_nonce (&seed, sizeof (seed));
+	srand (seed);
 }
 
 static const char HEXC[] = "0123456789ABCDEF";
@@ -875,6 +880,8 @@ gkr_crypto_skey_make_id (gcry_sexp_t s_key)
 {
 	guchar hash[20];
 	
+	g_return_val_if_fail (s_key != NULL, NULL);
+	
 	if (!gcry_pk_get_keygrip (s_key, hash))
 		g_return_val_if_reached (NULL);
 	
