@@ -77,6 +77,14 @@ enum {
 #define PAM_AUTHTOK_RECOVER_ERR PAM_AUTHTOK_RECOVERY_ERR
 #endif
 
+#ifndef PAM_EXTERN
+#ifdef PAM_STATIC
+#define PAM_EXTERN static
+#else
+#define PAM_EXTERN extern
+#endif
+#endif
+
 /* -----------------------------------------------------------------------------
  * HELPERS 
  */
@@ -271,9 +279,9 @@ static void
 setup_child (int inp[2], int outp[2], int errp[2], 
              pam_handle_t *ph, struct passwd *pwd, const char *password)
 {
-	char *args[] = { GNOME_KEYRING_DAEMON, "-d", "--login", NULL};
+	char *args[] = { GNOME_KEYRING_DAEMON, "--daemonize", "--login", NULL};
 	const char* display;
-	int ret;
+	int i, ret;
 	
 	assert (pwd);
 	assert (pwd->pw_dir);
@@ -290,6 +298,10 @@ setup_child (int inp[2], int outp[2], int errp[2],
 		        strerror (errno));
 		exit (EXIT_FAILURE);
 	}
+
+	/* Try valiantly to close unnecessary file descriptors */
+	for (i = STDERR; i < 64; ++i)
+		close (i);
 	    
 	/* Close unnecessary file descriptors */
 	close (inp[READ_END]);
@@ -860,9 +872,12 @@ pam_sm_open_session (pam_handle_t *ph, int flags, int argc, const char **argv)
 			return ret;
 	}
 
-	if (!started_daemon && password != NULL) {
-		if (unlock_keyring (ph, pwd, password) != PAM_SUCCESS)
-			return PAM_SERVICE_ERR;
+	/* If gnome keyring is running, but we didn't start it here, then unlock now */
+	if (get_any_env (ph, ENV_SOCKET) != NULL) {
+		if (!started_daemon && password != NULL) {
+			if (unlock_keyring (ph, pwd, password) != PAM_SUCCESS)
+				return PAM_SERVICE_ERR;
+		}
 	}
 	
 	return PAM_SUCCESS;
@@ -925,7 +940,7 @@ pam_chauthtok_preliminary (pam_handle_t *ph, struct passwd *pwd)
 }
 
 static int
-pam_chauthtok_update (pam_handle_t *ph, struct passwd *pwd)
+pam_chauthtok_update (pam_handle_t *ph, struct passwd *pwd, uint args)
 {
 	const char *password, *original;
 	int ret, started_daemon = 0;
@@ -967,6 +982,12 @@ pam_chauthtok_update (pam_handle_t *ph, struct passwd *pwd)
 		return ret;
 	
 	ret = change_keyring_password (ph, pwd, password, original);
+
+	/* if not auto_start, kill the daemon if we started it: we don't want
+	 * it to stay */
+	if (started_daemon && !(args & ARG_AUTO_START))
+		stop_daemon (ph, pwd);
+
 	if (ret != PAM_SUCCESS)
 		return ret;
 		
@@ -978,8 +999,11 @@ pam_sm_chauthtok (pam_handle_t *ph, int flags, int argc, const char **argv)
 {
 	const char *user;
 	struct passwd *pwd;
+	uint args;
 	int ret;
 	
+	args = parse_args (argc, argv);
+
 	/* Figure out and/or prompt for the user name */
 	ret = pam_get_user (ph, &user, NULL);
 	if (ret != PAM_SUCCESS) {
@@ -997,7 +1021,7 @@ pam_sm_chauthtok (pam_handle_t *ph, int flags, int argc, const char **argv)
 	if (flags & PAM_PRELIM_CHECK) 
 		return pam_chauthtok_preliminary (ph, pwd);
 	else if (flags & PAM_UPDATE_AUTHTOK)
-		return pam_chauthtok_update (ph, pwd);
+		return pam_chauthtok_update (ph, pwd, args);
 	else 
 		return PAM_IGNORE;
 }
